@@ -77,6 +77,9 @@ struct _ClutterBackendClass
   ClutterFeatureFlags (* get_features)     (ClutterBackend  *backend);
   void                (* redraw)           (ClutterBackend  *backend,
                                             ClutterStage    *stage);
+  gboolean            (* create_context)   (ClutterBackend  *backend,
+                                            gboolean         is_offscreen,
+                                            GError         **error);
   void                (* ensure_context)   (ClutterBackend  *backend,
                                             ClutterStage    *stage);
 
