@@ -76,14 +76,14 @@ cogl_material_new (void)
 {
   /* Create new - blank - material */
   CoglMaterial *material = g_new0 (CoglMaterial, 1);
-  GLfloat *unlit = material->unlit;
+  GLubyte *unlit = material->unlit;
   GLfloat *ambient = material->ambient;
   GLfloat *diffuse = material->diffuse;
   GLfloat *specular = material->specular;
   GLfloat *emission = material->emission;
 
   /* Use the same defaults as the GL spec... */
-  unlit[0] = 1.0; unlit[1] = 1.0; unlit[2] = 1.0; unlit[3] = 1.0;
+  unlit[0] = 0xff; unlit[1] = 0xff; unlit[2] = 0xff; unlit[3] = 0xff;
   material->flags |= COGL_MATERIAL_FLAG_DEFAULT_COLOR;
 
   /* Use the same defaults as the GL spec... */
@@ -111,9 +111,10 @@ cogl_material_new (void)
 #endif
   material->blend_src_factor_rgb = GL_ONE;
   material->blend_dst_factor_rgb = GL_ONE_MINUS_SRC_ALPHA;
-  material->flags |= COGL_MATERIAL_FLAG_DEFAULT_BLEND_FUNC;
+  material->flags |= COGL_MATERIAL_FLAG_DEFAULT_BLEND;
 
   material->layers = NULL;
+  material->n_layers = 0;
 
   return _cogl_material_handle_new (material);
 }
@@ -130,15 +131,24 @@ _cogl_material_free (CoglMaterial *material)
   g_free (material);
 }
 
-static void
-handle_automatic_blend_enable (CoglMaterial *material)
+static gboolean
+_cogl_material_needs_blending_enabled (CoglMaterial *material,
+                                       GLubyte      *override_color)
 {
   GList *tmp;
 
   /* XXX: If we expose manual control over ENABLE_BLEND, we'll add
    * a flag to know when it's user configured, so we don't trash it */
 
-  material->flags &= ~COGL_MATERIAL_FLAG_ENABLE_BLEND;
+  /* XXX: Uncomment this to disable all blending */
+#if 0
+  return;
+#endif
+
+  if ((override_color && override_color[3] != 0xff) ||
+      material->unlit[3] != 0xff)
+    return TRUE;
+
   for (tmp = material->layers; tmp != NULL; tmp = tmp->next)
     {
       CoglMaterialLayer *layer = tmp->data;
@@ -149,13 +159,46 @@ handle_automatic_blend_enable (CoglMaterial *material)
         continue;
 
       if (cogl_texture_get_format (layer->texture) & COGL_A_BIT)
-	material->flags |= COGL_MATERIAL_FLAG_ENABLE_BLEND;
+	return TRUE;
     }
 
-  if (material->unlit[3] != 1.0)
+  return FALSE;
+}
+
+static void
+handle_automatic_blend_enable (CoglMaterial *material)
+{
+  material->flags &= ~COGL_MATERIAL_FLAG_ENABLE_BLEND;
+
+  if (_cogl_material_needs_blending_enabled (material, NULL))
     material->flags |= COGL_MATERIAL_FLAG_ENABLE_BLEND;
 }
 
+/* If primitives have been logged in the journal referencing the current
+ * state of this material we need to flush the journal before we can
+ * modify it... */
+static void
+_cogl_material_pre_change_notify (CoglMaterial *material,
+                                  gboolean      only_color_change,
+                                  GLubyte      *new_color)
+{
+  /* XXX: We don't usually need to flush the journal just due to color changes
+   * since material colors are logged in the journals vertex buffer. The
+   * exception is when the change in color enables or disables the need for
+   * blending. */
+  if (only_color_change)
+    {
+      gboolean will_need_blending =
+        _cogl_material_needs_blending_enabled (material, new_color);
+      if (will_need_blending ==
+          (material->flags & COGL_MATERIAL_FLAG_ENABLE_BLEND) ? TRUE : FALSE)
+        return;
+    }
+
+  if (material->journal_ref_count)
+    _cogl_journal_flush ();
+}
+
 void
 cogl_material_get_color (CoglHandle  handle,
                          CoglColor  *color)
@@ -166,11 +209,20 @@ cogl_material_get_color (CoglHandle  handle,
 
   material = _cogl_material_pointer_from_handle (handle);
 
-  cogl_color_set_from_4f (color,
-                          material->unlit[0],
-                          material->unlit[1],
-                          material->unlit[2],
-                          material->unlit[3]);
+  cogl_color_set_from_4ub (color,
+                           material->unlit[0],
+                           material->unlit[1],
+                           material->unlit[2],
+                           material->unlit[3]);
+}
+
+/* This is used heavily by the cogl journal when logging quads */
+void
+_cogl_material_get_colorubv (CoglHandle  handle,
+                             guint8     *color)
+{
+  CoglMaterial *material = _cogl_material_pointer_from_handle (handle);
+  memcpy (color, material->unlit, 4);
 }
 
 void
@@ -178,26 +230,29 @@ cogl_material_set_color (CoglHandle       handle,
 			 const CoglColor *unlit_color)
 {
   CoglMaterial *material;
-  GLfloat       unlit[4];
+  GLubyte       unlit[4];
 
   g_return_if_fail (cogl_is_material (handle));
 
   material = _cogl_material_pointer_from_handle (handle);
 
-  unlit[0] = cogl_color_get_red_float (unlit_color);
-  unlit[1] = cogl_color_get_green_float (unlit_color);
-  unlit[2] = cogl_color_get_blue_float (unlit_color);
-  unlit[3] = cogl_color_get_alpha_float (unlit_color);
+  unlit[0] = cogl_color_get_red_byte (unlit_color);
+  unlit[1] = cogl_color_get_green_byte (unlit_color);
+  unlit[2] = cogl_color_get_blue_byte (unlit_color);
+  unlit[3] = cogl_color_get_alpha_byte (unlit_color);
   if (memcmp (unlit, material->unlit, sizeof (unlit)) == 0)
     return;
 
+  /* possibly flush primitives referencing the current state... */
+  _cogl_material_pre_change_notify (material, TRUE, unlit);
+
   memcpy (material->unlit, unlit, sizeof (unlit));
 
   material->flags &= ~COGL_MATERIAL_FLAG_DEFAULT_COLOR;
-  if (unlit[0] == 1.0 &&
-      unlit[1] == 1.0 &&
-      unlit[2] == 1.0 &&
-      unlit[3] == 1.0)
+  if (unlit[0] == 0xff &&
+      unlit[1] == 0xff &&
+      unlit[2] == 0xff &&
+      unlit[3] == 0xff)
     material->flags |= COGL_MATERIAL_FLAG_DEFAULT_COLOR;
 
   handle_automatic_blend_enable (material);
@@ -255,6 +310,9 @@ cogl_material_set_ambient (CoglHandle handle,
 
   material = _cogl_material_pointer_from_handle (handle);
 
+  /* possibly flush primitives referencing the current state... */
+  _cogl_material_pre_change_notify (material, FALSE, NULL);
+
   ambient = material->ambient;
   ambient[0] = cogl_color_get_red_float (ambient_color);
   ambient[1] = cogl_color_get_green_float (ambient_color);
@@ -292,6 +350,9 @@ cogl_material_set_diffuse (CoglHandle handle,
 
   material = _cogl_material_pointer_from_handle (handle);
 
+  /* possibly flush primitives referencing the current state... */
+  _cogl_material_pre_change_notify (material, FALSE, NULL);
+
   diffuse = material->diffuse;
   diffuse[0] = cogl_color_get_red_float (diffuse_color);
   diffuse[1] = cogl_color_get_green_float (diffuse_color);
@@ -337,6 +398,9 @@ cogl_material_set_specular (CoglHandle handle,
 
   material = _cogl_material_pointer_from_handle (handle);
 
+  /* possibly flush primitives referencing the current state... */
+  _cogl_material_pre_change_notify (material, FALSE, NULL);
+
   specular = material->specular;
   specular[0] = cogl_color_get_red_float (specular_color);
   specular[1] = cogl_color_get_green_float (specular_color);
@@ -372,6 +436,9 @@ cogl_material_set_shininess (CoglHandle handle,
 
   material = _cogl_material_pointer_from_handle (handle);
 
+  /* possibly flush primitives referencing the current state... */
+  _cogl_material_pre_change_notify (material, FALSE, NULL);
+
   material->shininess = (GLfloat)shininess * 128.0;
 
   material->flags &= ~COGL_MATERIAL_FLAG_DEFAULT_GL_MATERIAL;
@@ -405,6 +472,9 @@ cogl_material_set_emission (CoglHandle handle,
 
   material = _cogl_material_pointer_from_handle (handle);
 
+  /* possibly flush primitives referencing the current state... */
+  _cogl_material_pre_change_notify (material, FALSE, NULL);
+
   emission = material->emission;
   emission[0] = cogl_color_get_red_float (emission_color);
   emission[1] = cogl_color_get_green_float (emission_color);
@@ -424,6 +494,10 @@ cogl_material_set_alpha_test_function (CoglHandle handle,
   g_return_if_fail (cogl_is_material (handle));
 
   material = _cogl_material_pointer_from_handle (handle);
+
+  /* possibly flush primitives referencing the current state... */
+  _cogl_material_pre_change_notify (material, FALSE, NULL);
+
   material->alpha_func = alpha_func;
   material->alpha_func_reference = (GLfloat)alpha_reference;
 
@@ -571,6 +645,9 @@ cogl_material_set_blend (CoglHandle handle,
       a = &statements[1];
     }
 
+  /* possibly flush primitives referencing the current state... */
+  _cogl_material_pre_change_notify (material, FALSE, NULL);
+
 #ifndef HAVE_COGL_GLES
   setup_blend_state (rgb,
                      &material->blend_equation_rgb,
@@ -587,7 +664,7 @@ cogl_material_set_blend (CoglHandle handle,
                      &material->blend_dst_factor_rgb);
 #endif
 
-  material->flags &= ~COGL_MATERIAL_FLAG_DEFAULT_BLEND_FUNC;
+  material->flags &= ~COGL_MATERIAL_FLAG_DEFAULT_BLEND;
 
   return TRUE;
 }
@@ -604,13 +681,16 @@ cogl_material_set_blend_constant (CoglHandle handle,
 
   material = _cogl_material_pointer_from_handle (handle);
 
+  /* possibly flush primitives referencing the current state... */
+  _cogl_material_pre_change_notify (material, FALSE, NULL);
+
   constant = material->blend_constant;
   constant[0] = cogl_color_get_red_float (constant_color);
   constant[1] = cogl_color_get_green_float (constant_color);
   constant[2] = cogl_color_get_blue_float (constant_color);
   constant[3] = cogl_color_get_alpha_float (constant_color);
 
-  material->flags &= ~COGL_MATERIAL_FLAG_DEFAULT_BLEND_FUNC;
+  material->flags &= ~COGL_MATERIAL_FLAG_DEFAULT_BLEND;
 #endif
 }
 
@@ -644,6 +724,9 @@ _cogl_material_get_layer (CoglMaterial *material,
   if (!create_if_not_found)
     return NULL;
 
+  /* possibly flush primitives referencing the current state... */
+  _cogl_material_pre_change_notify (material, FALSE, NULL);
+
   layer = g_new0 (CoglMaterialLayer, 1);
 
   layer_handle = _cogl_material_layer_handle_new (layer);
@@ -682,20 +765,23 @@ cogl_material_set_layer (CoglHandle material_handle,
 {
   CoglMaterial	    *material;
   CoglMaterialLayer *layer;
-  int		     n_layers;
 
   g_return_if_fail (cogl_is_material (material_handle));
   g_return_if_fail (texture_handle == COGL_INVALID_HANDLE
                     || cogl_is_texture (texture_handle));
 
   material = _cogl_material_pointer_from_handle (material_handle);
+
   layer = _cogl_material_get_layer (material_handle, layer_index, TRUE);
 
   if (texture_handle == layer->texture)
     return;
 
-  n_layers = g_list_length (material->layers);
-  if (n_layers >= CGL_MAX_COMBINED_TEXTURE_IMAGE_UNITS)
+  /* possibly flush primitives referencing the current state... */
+  _cogl_material_pre_change_notify (material, FALSE, NULL);
+
+  material->n_layers = g_list_length (material->layers);
+  if (material->n_layers >= CGL_MAX_COMBINED_TEXTURE_IMAGE_UNITS)
     {
       if (!(material->flags & COGL_MATERIAL_FLAG_SHOWN_SAMPLER_WARNING))
 	{
@@ -856,6 +942,9 @@ cogl_material_set_layer_combine (CoglHandle handle,
       a = &statements[1];
     }
 
+  /* possibly flush primitives referencing the current state... */
+  _cogl_material_pre_change_notify (material, FALSE, NULL);
+
   setup_texture_combine_state (rgb,
                                &layer->texture_combine_rgb_func,
                                layer->texture_combine_rgb_src,
@@ -886,6 +975,9 @@ cogl_material_set_layer_combine_constant (CoglHandle handle,
   material = _cogl_material_pointer_from_handle (handle);
   layer = _cogl_material_get_layer (material, layer_index, TRUE);
 
+  /* possibly flush primitives referencing the current state... */
+  _cogl_material_pre_change_notify (material, FALSE, NULL);
+
   constant = layer->texture_combine_constant;
   constant[0] = cogl_color_get_red_float (constant_color);
   constant[1] = cogl_color_get_green_float (constant_color);
@@ -909,6 +1001,9 @@ cogl_material_set_layer_matrix (CoglHandle material_handle,
   material = _cogl_material_pointer_from_handle (material_handle);
   layer = _cogl_material_get_layer (material, layer_index, TRUE);
 
+  /* possibly flush primitives referencing the current state... */
+  _cogl_material_pre_change_notify (material, FALSE, NULL);
+
   layer->matrix = *matrix;
 
   layer->flags |= COGL_MATERIAL_LAYER_FLAG_DIRTY;
@@ -931,18 +1026,29 @@ cogl_material_remove_layer (CoglHandle material_handle,
   CoglMaterial	     *material;
   CoglMaterialLayer  *layer;
   GList		     *tmp;
+  gboolean            notified_change = FALSE;
 
   g_return_if_fail (cogl_is_material (material_handle));
 
   material = _cogl_material_pointer_from_handle (material_handle);
+
   for (tmp = material->layers; tmp != NULL; tmp = tmp->next)
     {
       layer = tmp->data;
       if (layer->index == layer_index)
 	{
 	  CoglHandle handle = (CoglHandle) layer;
+
+          /* possibly flush primitives referencing the current state... */
+          if (!notified_change)
+            {
+              _cogl_material_pre_change_notify (material, FALSE, NULL);
+              notified_change = TRUE;
+            }
+
 	  cogl_handle_unref (handle);
 	  material->layers = g_list_remove (material->layers, layer);
+          material->n_layers--;
 	  break;
 	}
     }
@@ -992,6 +1098,18 @@ cogl_material_get_layers (CoglHandle material_handle)
   return material->layers;
 }
 
+int
+cogl_material_get_n_layers (CoglHandle material_handle)
+{
+  CoglMaterial *material;
+
+  g_return_val_if_fail (cogl_is_material (material_handle), 0);
+
+  material = _cogl_material_pointer_from_handle (material_handle);
+
+  return material->n_layers;
+}
+
 CoglMaterialLayerType
 cogl_material_layer_get_type (CoglHandle layer_handle)
 {
@@ -1218,7 +1336,13 @@ _cogl_material_flush_layers_gl_state (CoglMaterial *material,
         (disable_mask & (1<<i)) ? TRUE : FALSE;
 
       tex_handle = layer->texture;
-      cogl_texture_get_gl_texture (tex_handle, &gl_texture, &gl_target);
+      if (tex_handle != COGL_INVALID_HANDLE)
+        cogl_texture_get_gl_texture (tex_handle, &gl_texture, &gl_target);
+      else
+        {
+          new_gl_layer_info.fallback = TRUE;
+          gl_target = GL_TEXTURE_2D;
+        }
 
       if (new_gl_layer_info.layer0_overridden)
         gl_texture = layer0_override_texture;
@@ -1318,7 +1442,10 @@ _cogl_material_flush_layers_gl_state (CoglMaterial *material,
                 !gl_layer_info->disabled))
 #endif
             {
+  /* XXX: Debug: Comment this out to disable all texturing: */
+#if 1
               GE (glEnable (gl_target));
+#endif
             }
         }
       else
@@ -1369,18 +1496,30 @@ _cogl_material_flush_layers_gl_state (CoglMaterial *material,
 }
 
 static void
-_cogl_material_flush_base_gl_state (CoglMaterial *material)
+_cogl_material_flush_base_gl_state (CoglMaterial *material,
+                                    gboolean skip_gl_color)
 {
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
-  if (!(ctx->current_material_flags & COGL_MATERIAL_FLAG_DEFAULT_COLOR
-        && material->flags & COGL_MATERIAL_FLAG_DEFAULT_COLOR))
+  /* XXX:
+   * Currently we only don't update state when the flags indicate that the
+   * current material uses the defaults, and the new material also uses the
+   * defaults, but we could do deeper comparisons of state. */
+
+  if (!skip_gl_color)
     {
-      /* GLES doesn't have glColor4fv... */
-      GE (glColor4f (material->unlit[0],
-                     material->unlit[1],
-                     material->unlit[2],
-                     material->unlit[3]));
+      if (!(ctx->current_material_flags & COGL_MATERIAL_FLAG_DEFAULT_COLOR
+            && material->flags & COGL_MATERIAL_FLAG_DEFAULT_COLOR) ||
+          /* Assume if we were previously told to skip the color, then
+           * the current color needs updating... */
+          ctx->current_material_flush_options.flags &
+          COGL_MATERIAL_FLUSH_SKIP_GL_COLOR)
+        {
+          GE (glColor4ub (material->unlit[0],
+                          material->unlit[1],
+                          material->unlit[2],
+                          material->unlit[3]));
+        }
     }
 
   if (!(ctx->current_material_flags & COGL_MATERIAL_FLAG_DEFAULT_GL_MATERIAL
@@ -1401,8 +1540,8 @@ _cogl_material_flush_base_gl_state (CoglMaterial *material)
       GE (glAlphaFunc (material->alpha_func, material->alpha_func_reference));
     }
 
-  if (!(ctx->current_material_flags & COGL_MATERIAL_FLAG_DEFAULT_BLEND_FUNC
-        && material->flags & COGL_MATERIAL_FLAG_DEFAULT_BLEND_FUNC))
+  if (!(ctx->current_material_flags & COGL_MATERIAL_FLAG_DEFAULT_BLEND
+        && material->flags & COGL_MATERIAL_FLAG_DEFAULT_BLEND))
     {
 #if defined (HAVE_COGL_GLES2)
       gboolean have_blend_equation_seperate = TRUE;
@@ -1441,32 +1580,33 @@ _cogl_material_flush_base_gl_state (CoglMaterial *material)
 }
 
 void
-_cogl_material_flush_gl_state (CoglHandle handle, ...)
+_cogl_material_flush_gl_state (CoglHandle handle,
+                               CoglMaterialFlushOptions *options)
 {
-  CoglMaterial           *material;
-  va_list                 ap;
-  CoglMaterialFlushOption option;
-  guint32                 fallback_layers = 0;
-  guint32                 disable_layers = 0;
-  GLuint                  layer0_override_texture = 0;
+  CoglMaterial  *material;
+  guint32        fallback_layers = 0;
+  guint32        disable_layers = 0;
+  GLuint         layer0_override_texture = 0;
+  gboolean       skip_gl_color = FALSE;
 
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
   material = _cogl_material_pointer_from_handle (handle);
 
-  _cogl_material_flush_base_gl_state (material);
-
-  va_start (ap, handle);
-  while ((option = va_arg (ap, CoglMaterialFlushOption)))
+  if (options)
     {
-      if (option == COGL_MATERIAL_FLUSH_FALLBACK_MASK)
-        fallback_layers = va_arg (ap, guint32);
-      else if (option == COGL_MATERIAL_FLUSH_DISABLE_MASK)
-        disable_layers = va_arg (ap, guint32);
-      else if (option == COGL_MATERIAL_FLUSH_LAYER0_OVERRIDE)
-        layer0_override_texture = va_arg (ap, GLuint);
+      if (options->flags & COGL_MATERIAL_FLUSH_FALLBACK_MASK)
+        fallback_layers = options->fallback_layers;
+      if (options->flags & COGL_MATERIAL_FLUSH_DISABLE_MASK)
+        disable_layers = options->disable_layers;
+      if (options->flags & COGL_MATERIAL_FLUSH_LAYER0_OVERRIDE)
+        layer0_override_texture = options->layer0_override_texture;
+      if (options->flags & COGL_MATERIAL_FLUSH_SKIP_GL_COLOR)
+        skip_gl_color = TRUE;
     }
-  va_end (ap);
+
+  _cogl_material_flush_base_gl_state (material,
+                                      skip_gl_color);
 
   _cogl_material_flush_layers_gl_state (material,
                                         fallback_layers,
@@ -1483,6 +1623,271 @@ _cogl_material_flush_gl_state (CoglHandle handle, ...)
 
   ctx->current_material = handle;
   ctx->current_material_flags = material->flags;
+  if (options)
+    ctx->current_material_flush_options = *options;
+  else
+    memset (&ctx->current_material_flush_options,
+            0, sizeof (CoglMaterialFlushOptions));
+}
+
+static gboolean
+_cogl_material_layer_equal (CoglMaterialLayer *material0_layer,
+                            CoglHandle         material0_layer_texture,
+                            CoglMaterialLayer *material1_layer,
+                            CoglHandle         material1_layer_texture)
+{
+  if (material0_layer_texture != material1_layer_texture)
+    return FALSE;
+
+  if ((material0_layer->flags & COGL_MATERIAL_LAYER_FLAG_DEFAULT_COMBINE) !=
+      (material1_layer->flags & COGL_MATERIAL_LAYER_FLAG_DEFAULT_COMBINE))
+    return FALSE;
+
+#if 0 /* TODO */
+  if (!_deep_are_layer_combines_equal ())
+    return FALSE;
+#else
+  if (!(material0_layer->flags & COGL_MATERIAL_LAYER_FLAG_DEFAULT_COMBINE))
+    return FALSE;
+#endif
+
+  return TRUE;
+}
+
+/* This is used by the Cogl journal to compare materials so that it
+ * can split up geometry that needs different OpenGL state.
+ *
+ * It is acceptable to have false negatives - although they will result
+ * in redundant OpenGL calls that try and update the state.
+ *
+ * False positives aren't allowed.
+ */
+gboolean
+_cogl_material_equal (CoglHandle material0_handle,
+                      CoglMaterialFlushOptions *material0_flush_options,
+                      CoglHandle material1_handle,
+                      CoglMaterialFlushOptions *material1_flush_options)
+{
+  CoglMaterial          *material0;
+  CoglMaterial          *material1;
+  CoglMaterialFlushFlag  flush_flags0 = material0_flush_options->flags;
+  CoglMaterialFlushFlag  flush_flags1 = material1_flush_options->flags;
+  guint32                fallback_layers0;
+  guint32                fallback_layers1;
+  guint32                disable_layers0;
+  guint32                disable_layers1;
+  GList                 *l0, *l1;
+  int                    i;
+
+  /* Compare the flush options first; if they are equivalent then we
+   * can potentially return quickly if the material handles then match. */
+
+
+  /* The skip color option is used when the color of the material is being
+   * submitted in a vertex array so cogl_material_flush_gl_state doesn't
+   * need to call glColor.
+   * - A skip gl color material following a non skip color material doesn't
+   *   need a state change since putting a color in a vertex array (as done
+   *   for skip color materials) would simply take precedence over one
+   *   previously specified by glColor (as done for non skip color materials)
+   * - A non skip color material following a skip color material also doesn't
+   *   need a state change for the same reason.
+   * - The problem is that a non skip color, followed by a skip color, followed
+   *   by a non skip color does require a state change. Since we don't have
+   *   enough contextual information here we currently return FALSE whenever
+   *   the skip color option changes. */
+  if ((flush_flags0 & COGL_MATERIAL_FLUSH_SKIP_GL_COLOR) !=
+      (flush_flags1 & COGL_MATERIAL_FLUSH_SKIP_GL_COLOR))
+    return FALSE;
+
+  fallback_layers0 = flush_flags0 & COGL_MATERIAL_FLUSH_FALLBACK_MASK ?
+    material0_flush_options->fallback_layers : 0;
+  fallback_layers1 = flush_flags1 & COGL_MATERIAL_FLUSH_FALLBACK_MASK ?
+    material1_flush_options->fallback_layers : 0;
+  if (fallback_layers0 != fallback_layers1)
+    return FALSE;
+
+  disable_layers0 = flush_flags0 & COGL_MATERIAL_FLUSH_DISABLE_MASK ?
+    material0_flush_options->disable_layers : 0;
+  disable_layers1 = flush_flags1 & COGL_MATERIAL_FLUSH_DISABLE_MASK ?
+    material1_flush_options->disable_layers : 0;
+  if (disable_layers0 != disable_layers1)
+    return FALSE;
+
+  /* NB: Some unlikely false negatives are possible here. */
+  if ((flush_flags0 & COGL_MATERIAL_FLUSH_LAYER0_OVERRIDE) !=
+      (flush_flags1 & COGL_MATERIAL_FLUSH_LAYER0_OVERRIDE))
+    return FALSE;
+
+  if ((flush_flags0 & COGL_MATERIAL_FLUSH_LAYER0_OVERRIDE) &&
+      material0_flush_options->layer0_override_texture !=
+      material1_flush_options->layer0_override_texture)
+    return FALSE;
+
+  /* Since we know the flush options match at this point, if the material
+   * handles match then we know they are equivalent. */
+  if (material0_handle == material1_handle)
+    return TRUE;
+
+  /* Now we need to look in more detail... */
+
+  material0 = _cogl_material_pointer_from_handle (material0_handle);
+  material1 = _cogl_material_pointer_from_handle (material1_handle);
+
+  if (!(material0_flush_options->flags & COGL_MATERIAL_FLUSH_SKIP_GL_COLOR) &&
+      !memcmp (material0->unlit, material1->unlit, sizeof (material0->unlit)))
+    return FALSE;
+
+  /* First we simply try and find a difference according to default flags
+   * for each material component to avoid deeper comparison. */
+
+  if ((material0->flags & COGL_MATERIAL_FLAG_DEFAULT_GL_MATERIAL) !=
+      (material1->flags & COGL_MATERIAL_FLAG_DEFAULT_GL_MATERIAL))
+    return FALSE;
+
+  if ((material0->flags & COGL_MATERIAL_FLAG_DEFAULT_ALPHA_FUNC) !=
+      (material1->flags & COGL_MATERIAL_FLAG_DEFAULT_ALPHA_FUNC))
+    return FALSE;
+
+  /* Potentially blending could be "enabled" but the blend mode
+   * could be equivalent to being disabled, but we accept those false
+   * negatives for now. */
+  if ((material0->flags & COGL_MATERIAL_FLAG_ENABLE_BLEND) !=
+      (material1->flags & COGL_MATERIAL_FLAG_ENABLE_BLEND))
+    return FALSE;
+
+  if ((material0->flags & COGL_MATERIAL_FLAG_ENABLE_BLEND) &&
+      (material0->flags & COGL_MATERIAL_FLAG_DEFAULT_BLEND) !=
+      (material1->flags & COGL_MATERIAL_FLAG_DEFAULT_BLEND))
+    return FALSE;
+
+  /* If we still haven't found a difference then do a deeper comparison..
+   *
+   * Actually we don't currently do this; we simply assume anything
+   * non default is different and accept the false negatives for now.
+   */
+
+#if 0 /* TODO */
+  if (!_deep_are_gl_materials_equal ())
+    return FALSE;
+#else
+  /* Just assume that all non default materials are different */
+  if (!(material0->flags & COGL_MATERIAL_FLAG_DEFAULT_GL_MATERIAL))
+    return FALSE;
+#endif
+
+#if 0 /* TODO */
+  if (!_deep_are_alpha_funcs_equal ())
+    return FALSE;
+#else
+  /* Just assume that all non default alpha funcs are different */
+  if (!(material0->flags & COGL_MATERIAL_FLAG_DEFAULT_ALPHA_FUNC))
+    return FALSE;
+#endif
+
+  if (material0->flags & COGL_MATERIAL_FLAG_ENABLE_BLEND)
+    {
+#if 0 /* TODO */
+      if (!_deep_is_blend_equal ())
+        return FALSE;
+#else
+      if (!(material0->flags & COGL_MATERIAL_FLAG_DEFAULT_BLEND))
+        return FALSE;
+#endif
+    }
+
+
+  /* Finally compare each of the material layers ... */
+
+  l0 = material0->layers;
+  l1 = material1->layers;
+  i = 0;
+
+  /* NB: At this point we know if COGL_MATERIAL_FLUSH_LAYER0_OVERRIDE is being
+   * used then both materials are overriding with the same texture. */
+  if (flush_flags0 & COGL_MATERIAL_FLUSH_LAYER0_OVERRIDE &&
+      l0 && l1)
+    {
+      /* We still need to check if the combine modes etc are equal, but we
+       * simply pass COGL_INVALID_HANDLE for both texture handles so they will
+       * be considered equal */
+      if (!_cogl_material_layer_equal (l0->data, COGL_INVALID_HANDLE,
+                                       l1->data, COGL_INVALID_HANDLE))
+        return FALSE;
+
+      l0 = l0->next;
+      l1 = l1->next;
+      i++;
+    }
+
+  while (l0 && l1)
+    {
+      CoglMaterialLayer *m0_layer;
+      CoglMaterialLayer *m1_layer;
+
+      if ((l0 == NULL && l1 != NULL) ||
+          (l1 == NULL && l0 != NULL))
+        return FALSE;
+
+      /* NB: At this point we know that the fallback and disable masks for
+       * both materials are equal. */
+      if (disable_layers0 & (1<<i))
+        goto next_layer;
+
+      m0_layer = l0->data;
+      m1_layer = l1->data;
+
+      /* NB: The use of a fallback texture doesn't imply that the combine
+       * modes etc are the same.
+       */
+      if ((disable_layers0 & (1<<i)) || (fallback_layers0 & (1<<i)))
+        {
+          /* As with layer0 overrides, we simply pass COGL_INVALID_HANDLEs for
+           * both texture handles here so they will be considered equal. */
+          if (!_cogl_material_layer_equal (m0_layer, COGL_INVALID_HANDLE,
+                                           m1_layer, COGL_INVALID_HANDLE))
+            return FALSE;
+        }
+      else
+        {
+          if (!_cogl_material_layer_equal (m0_layer, m0_layer->texture,
+                                           m1_layer, m1_layer->texture))
+            return FALSE;
+        }
+
+next_layer:
+      l0 = l0->next;
+      l1 = l1->next;
+      i++;
+    }
+
+  if ((l0 == NULL && l1 != NULL) ||
+      (l1 == NULL && l0 != NULL))
+    return FALSE;
+
+  return TRUE;
+}
+
+/* While a material is referenced by the Cogl journal we can not allow
+ * modifications, so this gives us a mechanism to track journal
+ * references separately */
+CoglHandle
+_cogl_material_journal_ref (CoglHandle material_handle)
+{
+  CoglMaterial *material =
+    material = _cogl_material_pointer_from_handle (material_handle);
+  material->journal_ref_count++;
+  cogl_handle_ref (material_handle);
+  return material_handle;
+}
+
+void
+_cogl_material_journal_unref (CoglHandle material_handle)
+{
+  CoglMaterial *material =
+    material = _cogl_material_pointer_from_handle (material_handle);
+  material->journal_ref_count--;
+  cogl_handle_unref (material_handle);
 }
 
 /* TODO: Should go in cogl.c, but that implies duplication which is also
@@ -1509,9 +1914,10 @@ cogl_set_source (CoglHandle material_handle)
 void
 cogl_set_source_texture (CoglHandle texture_handle)
 {
-  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
   CoglColor white;
 
+  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
+
   g_return_if_fail (texture_handle != COGL_INVALID_HANDLE);
 
   cogl_material_set_layer (ctx->default_material, 0, texture_handle);
@@ -1558,6 +1964,9 @@ cogl_material_set_layer_filters (CoglHandle         handle,
   material = _cogl_material_pointer_from_handle (handle);
   layer = _cogl_material_get_layer (material, layer_index, TRUE);
 
+  /* possibly flush primitives referencing the current state... */
+  _cogl_material_pre_change_notify (material, FALSE, NULL);
+
   layer->min_filter = min_filter;
   layer->mag_filter = mag_filter;
 }
