@@ -39,6 +39,7 @@
 #include "cogl-material.h"
 #include "cogl-context.h"
 #include "cogl-handle.h"
+#include "cogl-primitives.h"
 
 #include <string.h>
 #include <stdlib.h>
@@ -60,8 +61,6 @@
 
 #endif
 
-extern void _cogl_journal_flush (void);
-
 static void _cogl_texture_free (CoglTexture *tex);
 
 COGL_HANDLE_DEFINE (Texture, texture);
@@ -397,7 +396,7 @@ _cogl_texture_download_from_gl (CoglTexture *tex,
 	  if (y_span->waste != 0 || x_span->waste != 0)
 	    {
 	      /* Setup temp bitmap for slice subregion */
-	      slice_bmp.format = tex->bitmap.format;
+	      slice_bmp.format = target_bmp->format;
 	      slice_bmp.width  = x_span->size;
 	      slice_bmp.height = y_span->size;
 	      slice_bmp.rowstride = bpp * slice_bmp.width;
